-- DROP SEQUENCE SIMPLE_SEQUENCE IF EXISTS CASCADE;
DROP TABLE AUTH_USER IF EXISTS CASCADE;
DROP TABLE AUTH_ROLE IF EXISTS CASCADE;
DROP TABLE USER_ROLE IF EXISTS CASCADE;
DROP TABLE USER_SESSION IF EXISTS CASCADE;

-- CREATE SEQUENCE SIMPLE_SEQUENCE AS BIGINT START WITH 100 INCREMENT BY 1;

CREATE TABLE AUTH_USER (
  ID BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL
, USERNAME VARCHAR(30) NOT NULL
, PASSWORD VARCHAR(50) NULL
, NAME VARCHAR(100) NOT NULL 
, EMAIL VARCHAR(100) NULL
, LAST_LOGIN TIMESTAMP(0) NULL
, VERSION INT DEFAULT 0 NOT NULL
);

CREATE INDEX IX_AUTH_USER_USERNAME ON AUTH_USER (USERNAME);

ALTER TABLE AUTH_USER ADD CONSTRAINT PK_AUTH_USER
	PRIMARY KEY (ID)
;

CREATE TABLE AUTH_ROLE (
  ID BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL
, NAME VARCHAR(100) NOT NULL 
, DESCRIPTION VARCHAR(100) NULL
, VERSION INT DEFAULT 0 NOT NULL
);

ALTER TABLE AUTH_ROLE ADD CONSTRAINT PK_AUTH_ROLE
	PRIMARY KEY (ID)
;

CREATE TABLE USER_ROLE (
  ID BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL
, AUTH_USER_ID BIGINT NOT NULL
, AUTH_ROLE_ID BIGINT NOT NULL
, VERSION INT DEFAULT 0 NOT NULL
);

ALTER TABLE USER_ROLE ADD CONSTRAINT PK_USER_ROLE
	PRIMARY KEY (ID)
;

ALTER TABLE USER_ROLE ADD CONSTRAINT FK_USER_ROLE_AUTH_USER
	FOREIGN KEY (AUTH_USER_ID) REFERENCES AUTH_USER (ID) ON DELETE CASCADE
;

ALTER TABLE USER_ROLE ADD CONSTRAINT FK_USER_ROLE_AUTH_ROLE
	FOREIGN KEY (AUTH_ROLE_ID) REFERENCES AUTH_ROLE (ID) ON DELETE CASCADE
;

CREATE INDEX IX_USER_ROLE_USER_ID ON USER_ROLE (AUTH_USER_ID);

CREATE INDEX IX_USER_ROLE_ROLE_ID ON USER_ROLE (AUTH_ROLE_ID);

CREATE TABLE USER_SESSION (
  ID BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL
, AUTH_USER_ID BIGINT NOT NULL
, LAST_ACCESS TIMESTAMP(0) DEFAULT CURRENT_TIMESTAMP NOT NULL
)
;

ALTER TABLE USER_SESSION ADD CONSTRAINT PK_USER_SESSION
	PRIMARY KEY (ID)
;

ALTER TABLE USER_SESSION ADD CONSTRAINT FK_USER_SESSION_AUTH_USER
	FOREIGN KEY (AUTH_USER_ID) REFERENCES AUTH_USER (ID) ON DELETE CASCADE
;

CREATE INDEX IX_USER_SESSION_ROLE_ID ON USER_SESSION (AUTH_USER_ID);
