DROP TABLE IF EXISTS CONTACT CASCADE;
DROP TABLE IF EXISTS PERSON CASCADE;
DROP TABLE IF EXISTS COUNTRY CASCADE;

CREATE TABLE PERSON (
  ID BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  FIRST_NAME VARCHAR(100) NOT NULL,
  LAST_NAME VARCHAR(100) NOT NULL,
  DATE_OF_BIRTH DATE,
  GENDER ENUM('M','F','0') NOT NULL,
  SSN VARCHAR(100)
);

ALTER TABLE PERSON ADD COLUMN VERSION INT NOT NULL DEFAULT 0; 

CREATE  INDEX IX_PERSON_LAST_NAME ON PERSON (LAST_NAME);

CREATE TABLE RELATIVE (
  ID BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  PERSON_ID BIGINT NOT NULL,
  REL_PERSON_ID BIGINT NOT NULL,
  RTYPE ENUM('F', 'M', 'S', 'D', 'H', 'W', 'B', 'I') NOT NULL 
);

ALTER TABLE RELATIVE ADD CONSTRAINT FK_RELATIVE_PERSON
	FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE
;

ALTER TABLE RELATIVE ADD CONSTRAINT FK_RELATIVE_REL_PERSON
	FOREIGN KEY (REL_PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE
;

CREATE TABLE COUNTRY (
  ID BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  CODE CHAR(2) NOT NULL,
  NAME VARCHAR(100)
);

CREATE UNIQUE INDEX IX_COUNTRY_CODE ON COUNTRY (CODE);
CREATE INDEX IX_COUNTRY_NAME ON COUNTRY (NAME);

CREATE TABLE CONTACT (
  ID BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  PERSON_ID BIGINT NOT NULL,
  CTYPE ENUM('0','1','2'), 
  ADDRESS VARCHAR(100),
  PHONE_NUMBER VARCHAR(100),
  COUNTRY_CODE CHAR(2) NOT NULL
);

ALTER TABLE CONTACT ADD COLUMN VERSION INT NOT NULL DEFAULT 0; 

ALTER TABLE CONTACT ADD CONSTRAINT FK_CONTACT_PERSON
	FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE;
	
ALTER TABLE CONTACT ADD CONSTRAINT FK_CONTACT_COUNTRY 
    FOREIGN KEY (COUNTRY_CODE) REFERENCES COUNTRY (CODE);
	
INSERT INTO COUNTRY (ID,CODE,NAME) values (1,'BE','Belgium');
INSERT INTO COUNTRY (ID,CODE,NAME) values (2,'BG','Bulgaria');
INSERT INTO COUNTRY (ID,CODE,NAME) values (3,'CZ','Czech Republic');
INSERT INTO COUNTRY (ID,CODE,NAME) values (4,'DK','Denmark');
INSERT INTO COUNTRY (ID,CODE,NAME) values (5,'DE','Germany');
INSERT INTO COUNTRY (ID,CODE,NAME) values (6,'EE','Estonia');
INSERT INTO COUNTRY (ID,CODE,NAME) values (7,'IE','Ireland');
INSERT INTO COUNTRY (ID,CODE,NAME) values (8,'EL','Greece');
INSERT INTO COUNTRY (ID,CODE,NAME) values (9,'ES','Spain');
INSERT INTO COUNTRY (ID,CODE,NAME) values (10,'FR','France');
INSERT INTO COUNTRY (ID,CODE,NAME) values (11,'IT','Italy');
INSERT INTO COUNTRY (ID,CODE,NAME) values (12,'CY','Cyprus');
INSERT INTO COUNTRY (ID,CODE,NAME) values (13,'LV','Latvia');
INSERT INTO COUNTRY (ID,CODE,NAME) values (14,'LT','Lithuania');
INSERT INTO COUNTRY (ID,CODE,NAME) values (15,'LU','Luxembourg');
INSERT INTO COUNTRY (ID,CODE,NAME) values (16,'HU','Hungary');
INSERT INTO COUNTRY (ID,CODE,NAME) values (17,'MT','Malta');
INSERT INTO COUNTRY (ID,CODE,NAME) values (18,'NL','Netherlands');
INSERT INTO COUNTRY (ID,CODE,NAME) values (19,'AT','Austria');
INSERT INTO COUNTRY (ID,CODE,NAME) values (20,'PL','Poland');
INSERT INTO COUNTRY (ID,CODE,NAME) values (21,'PT','Portugal');
INSERT INTO COUNTRY (ID,CODE,NAME) values (22,'RO','Romania');
INSERT INTO COUNTRY (ID,CODE,NAME) values (23,'SI','Slovenia');
INSERT INTO COUNTRY (ID,CODE,NAME) values (24,'SK','Slovakia');
INSERT INTO COUNTRY (ID,CODE,NAME) values (25,'FI','Finland');
INSERT INTO COUNTRY (ID,CODE,NAME) values (26,'SE','Sweden');
INSERT INTO COUNTRY (ID,CODE,NAME) values (27,'UK','United Kingdom');
commit;