DROP SEQUENCE SIMPLE_SEQUENCE
DROP TABLE CONTACT
DROP TABLE PERSON
DROP PROCEDURE NEW_PERSON;
DROP PROCEDURE NEW_PERSON_RET;
DROP FUNCTION AN_HOUR_BEFORE;

CREATE SEQUENCE SIMPLE_SEQUENCE START WITH 100 INCREMENT BY 1

CREATE TABLE PERSON
(
  ID BIGINT PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY
, FIRST_NAME VARCHAR(100) NOT NULL 
, LAST_NAME VARCHAR(100) NOT NULL 
, DATE_OF_BIRTH DATE
, GENDER VARCHAR(1) NOT NULL CONSTRAINT CHECK_PERSON_GENDER CHECK (GENDER IN ('M', 'F'))
, SSN VARCHAR(100) 
)

CREATE  INDEX IX_PERSON_LAST_NAME ON PERSON (LAST_NAME)

CREATE TABLE CONTACT
(
  ID BIGINT PRIMARY KEY GENERATED BY DEFAULT AS IDENTITY
, PERSON_ID BIGINT NOT NULL 
, TYPE INT NOT NULL CONSTRAINT CHECK_CONTACT_TYPE CHECK (TYPE IN (0, 1))
, ADDRESS VARCHAR(100) NOT NULL 
, PHONE_NUMBER VARCHAR(100)
)

ALTER TABLE CONTACT ADD CONSTRAINT FK_CONTACT_PERSON
	FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE


CREATE OR REPLACE PROCEDURE NEW_PERSON(OUT newid BIGINT, IN date_of_birth DATE, IN ssn VARCHAR(20), IN first_name VARCHAR(100), IN last_name VARCHAR(100), IN gender VARCHAR(1))
    LANGUAGE SQL
BEGIN
  INSERT INTO PERSON (DATE_OF_BIRTH, SSN, FIRST_NAME, LAST_NAME, GENDER) 
    VALUES (date_of_birth, ssn, first_name, last_name, gender);
  SET newid = IDENTITY_VAL_LOCAL();
END

CREATE OR REPLACE PROCEDURE NEW_PERSON_RET_RS(IN date_of_birth DATE, IN ssn VARCHAR(20), IN first_name VARCHAR(100), IN last_name VARCHAR(100), IN gender VARCHAR(1))
    DYNAMIC RESULT SETS 1
    LANGUAGE SQL
    MODIFIES SQL DATA
    NOT DETERMINISTIC
BEGIN
  DECLARE newid INTEGER;
  DECLARE C1 CURSOR WITH RETURN FOR SELECT * FROM PERSON WHERE ID = newid;
  INSERT INTO PERSON (DATE_OF_BIRTH, SSN, FIRST_NAME, LAST_NAME, GENDER) 
    VALUES (date_of_birth, ssn, first_name, last_name, gender);
  SET newid = IDENTITY_VAL_LOCAL();
  OPEN C1;
END

CREATE OR REPLACE FUNCTION AN_HOUR_BEFORE(IN t TIMESTAMP)
    RETURNS TIMESTAMP
    LANGUAGE SQL
    CONTAINS SQL
    NO EXTERNAL ACTION
    DETERMINISTIC
  RETURN t - 1 HOUR
