DROP TABLE IF EXISTS CONTACT CASCADE;
DROP TABLE IF EXISTS PERSON CASCADE;
DROP PROCEDURE IF EXISTS NEW_PERSON;
DROP PROCEDURE IF EXISTS NEW_PERSON_RET_RS;
DROP FUNCTION IF EXISTS AN_HOUR_BEFORE;

CREATE TABLE PERSON (
  ID BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  FIRST_NAME VARCHAR(100) NOT NULL,
  LAST_NAME VARCHAR(100) NOT NULL,
  DATE_OF_BIRTH DATE,
  GENDER ENUM('M','F','0') NOT NULL,
  SSN VARCHAR(100)
);

CREATE  INDEX IX_PERSON_LAST_NAME ON PERSON (LAST_NAME);

CREATE TABLE CONTACT (
  ID BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  PERSON_ID BIGINT NOT NULL,
  CTYPE ENUM('0','1','2'), 
  ADDRESS VARCHAR(100),
  PHONE_NUMBER VARCHAR(100)
);

ALTER TABLE CONTACT ADD CONSTRAINT FK_CONTACT_PERSON
	FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE;

CREATE PROCEDURE NEW_PERSON(OUT newid INTEGER, IN date_of_birth DATE, IN ssn VARCHAR(20), IN first_name VARCHAR(100), IN last_name VARCHAR(100), IN gender VARCHAR(1))
  BEGIN
    INSERT INTO PERSON (FIRST_NAME, LAST_NAME, DATE_OF_BIRTH, GENDER, SSN) 
    VALUES (first_name, last_name, date_of_birth, gender, ssn);
    SELECT last_insert_id() INTO newid;
  END

CREATE PROCEDURE NEW_PERSON_RET_RS(IN date_of_birth DATE, IN ssn VARCHAR(20), IN first_name VARCHAR(100), IN last_name VARCHAR(100), IN gender VARCHAR(1))
  BEGIN
    DECLARE temp_id INTEGER;
    INSERT INTO PERSON (FIRST_NAME, LAST_NAME, DATE_OF_BIRTH, GENDER, SSN) 
    VALUES (first_name, last_name, date_of_birth, gender, ssn);
    SELECT last_insert_id() INTO temp_id;
    SELECT * FROM PERSON WHERE ID = temp_id;
  END

CREATE FUNCTION AN_HOUR_BEFORE(t TIMESTAMP) RETURNS TIMESTAMP
BEGIN
      RETURN SUBTIME(t, '1:00:00.000000');
END
